--[[
Profit - Automatically sells low quality gray items and selected white items to the vendor.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Profit = LibStub( 'AceAddon-3.0'):NewAddon( 'AutoProfitY', 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0')
if not Profit then return end -- already loaded and no upgrade necessary

local L    = LibStub( 'AceLocale-3.0'):GetLocale( 'AutoProfitY')
local LibG = LibStub( 'LibGratuity-3.0')

Profit.name, Profit.localizedname = GetAddOnInfo( 'AutoProfitY')
Profit.version = format( '%s: %s |cFF20FF20(Cata)|r', L.Version, GetAddOnMetadata( 'AutoProfitY', 'Version') or 'unknown')

local DEFAULTS = { 
	profile = {
		enabled    = true,
		autosell   = false,
		silent     = false,
		showtotal  = true,
		soulbound  = false,
		spin       = 'profit',
		exceptions = {}
	},
	realm = {
		exceptions = {}
	}
}
local CLASS_MATCH = ITEM_CLASSES_ALLOWED:format( '([%w%s,]*)')
local PROFICIENCIES = {}
local DEFAULT_SPIN_RATE = 0.6

local button = LibStub( 'AceGUI-3.0'):Create( 'ProfitButton') -- ProfitModel, ProfitButton
local rotation = 0
local rotrate = 0

--[[---------------------------------------------------------------------------------
Local
------------------------------------------------------------------------------------]]
local function Coins( money)
	money = math.floor( tonumber( money) or 0)
	local g = math.floor( money / 10000)
	local s = math.floor( money % 10000 / 100)
	local c = money % 100
	if g > 0 then
		return format( '|cFFFFD100%dg|cFF000000.|cFFE6E6E6%02ds|cFF000000.|cFFC8602C%02dc|r', g, s, c)
	end
	if s > 0 then
		return format( '|cFFE6E6E6%ds|cFF000000.|cFFC8602C%02dc|r', s, c)
	end
	return format( '|cFFC8602C%dc|r', c)
end

local function IsSoulboundJunk( bagID, slotID, link)
	Profit:Debug( 'IsSoulboundJunk', link)
	if not Profit.db.profile.soulbound then
		return false 
	end
	LibG:SetBagItem( bagID, slotID)
	if not LibG:Find( ITEM_SOULBOUND) then
		return false
	end
	Profit:Debug( 'IsSoulboundJunk', 'not soulbound')
	local _, _, classes = LibG:Find( CLASS_MATCH)
	if classes then
		Profit:Debug( 'IsSoulboundJunk', 'class match')
		local _, class = UnitClass( 'player')
		local male = LOCALIZED_CLASS_NAMES_MALE[class]
		if not classes:match( male) then
			return true
		end
		Profit:Debug( 'IsSoulboundJunk', 'not my class')
	end
	local _, _, _, _, _, iType, iSubType, _, iEquipLoc = GetItemInfo( link)
	local tbl = PROFICIENCIES[iType]
	return tbl and (tbl[iSubType] or (tbl.noOffhand and iEquipLoc == 'INVTYPE_WEAPONOFFHAND'))
end

local function IsJunk( link)
	local _, _, quality = GetItemInfo( link)
	local id = link:match( 'item:(%d+)')
	if Profit.db.profile.exceptions[id] then
		return quality ~= 0
 	end
	if Profit.db.realm.exceptions[id] then
		return quality ~= 0
 	end
	return quality == 0
end

local function GetItemProfit( bagID, slotID, link)
	local _, itemCount = GetContainerItemInfo( bagID, slotID)
	local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo( link)
	return itemCount * (tonumber( itemSellPrice) or 0)
end

local function GetProfit()
	local profit = 0
	for bagID = 0,NUM_BAG_SLOTS do
		for slotID = 1,GetContainerNumSlots( bagID) do
			local link = GetContainerItemLink( bagID, slotID)
			if link and (IsJunk( link) or IsSoulboundJunk( bagID, slotID, link)) then
				profit = profit + GetItemProfit( bagID, slotID, link)
			end
		end
	end
	return profit
end

local function SellJunk()
	local profit = 0
	for bagID = 0,NUM_BAG_SLOTS do
		for slotID = 1,GetContainerNumSlots( bagID) do
			local link = GetContainerItemLink( bagID, slotID)
			if link and (IsJunk( link) or IsSoulboundJunk( bagID, slotID, link)) then
				profit = profit + GetItemProfit( bagID, slotID, link)
				UseContainerItem( bagID, slotID)
				if Profit.db.profile.report then
					Profit:Print( L.Sold:format( link))
				end
			end
		end
	end
	if profit > 0 and Profit.db.profile.profit then
		Profit:Print( L.Total:format( Coins( profit)))
	end
end

--[[---------------------------------------------------------------------------------
Script
------------------------------------------------------------------------------------]]
local function Button_OnEnter( button)
	Profit:Debug( 'Button_OnEnter')
	GameTooltip:SetOwner( button.frame, 'ANCHOR_RIGHT')
	GameTooltip:SetText( L.SellJunk)
	if Profit:IsSpin( 'over') then
		Profit:SetRate()
	elseif Profit:IsSpin( 'overProfit') then
		local profit = GetProfit()
		if profit > 0 then
			Profit:SetRate()
			SetTooltipMoney( GameTooltip, profit)
		else
			GameTooltip:AddLine( L.NoJunk, 1.0, 1.0, 1.0, 1)
		end
	end
	GameTooltip:Show()
end

local function Button_OnLeave( button)
	Profit:Debug( 'Button_OnLeave')
	GameTooltip:Hide()
	if Profit:IsSpin( 'over') then
		Profit:SetRate( 0)
	end
end

local function Button_OnClick( button, event)
	if MerchantFrame.selectedTab == 1 then
		GameTooltip:Hide()
		SellJunk()
		if Profit:IsSpin( 'profit') or Profit:IsSpin( 'overProfit') then
			Profit:SetRate( 0)
		end
	end
end

local function Button_OnUpdate( frame, elapsed)
	if rotrate > 0 then
		rotation = rotation + (elapsed * rotrate)
		button:SetRotation( rotation)
	else
		button:SetRotation()
	end
end

--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Profit:OnInitialize()
	self.db = LibStub( 'AceDB-3.0'):New( 'ProfitDB', DEFAULTS, 'Default')
	self.db.RegisterCallback( self, 'OnNewProfile')
	self.db.RegisterCallback( self, 'OnProfileChanged')
	self.db.RegisterCallback( self, 'OnProfileCopied')
	self.db.RegisterCallback( self, 'OnProfileReset')
	self.db.RegisterCallback( self, 'OnProfileDeleted')
	button:SetWidth( 33)
	button:SetHeight( 33)
	button:SetCallback( 'OnEnter',  Button_OnEnter)
	button:SetCallback( 'OnLeave',  Button_OnLeave)
	button:SetCallback( 'OnClick',  Button_OnClick)
end

function Profit:OnEnable()
	self:ToggleDebugLog( false)
	self:RegisterEvent( 'MERCHANT_SHOW')
	self:RegisterMessage( 'AUTO_PROFIT_UPDATE')
	self:Debug( 'AutoProfitY enabled')
	self:Print( self.version, 'loaded')
end

function Profit:OnDisable()
	self:UnregisterEvent( 'MERCHANT_SHOW')
	self:UnregisterMessage( 'AUTO_PROFIT_UPDATE')
	self:Debug( 'AutoProfitY disabled')
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
function Profit:SetDebug( debug)
	self:ToggleDebugLog( debug)
	self:SendMessage( 'AUTO_PROFIT_DEBUG', debug)
end

function Profit:MERCHANT_SHOW()
	self:Debug( 'MERCHANT_SHOW')
	self:SetRate( 0)
	button.frame:SetParent( MerchantFrame)
	button:ClearAllPoints()
	button:SetPoint( 'TOPRIGHT', MerchantFrame, 'TOPRIGHT', -41, -37)
	button:SetImage( 'Interface\\Icons\\INV_Misc_Coin_02')
--	button:SetModel( 'World\\Generic\\PassiveDoodads\\TreasurePiles\\GoldPileLarge01.m2')
	self:AUTO_PROFIT_UPDATE()
	button.frame:SetScript( 'OnUpdate', Button_OnUpdate)
	if self.db.profile.autosell then
		SellJunk()
	else
		self:RegisterEvent( 'BAG_UPDATE' )
		self:RegisterEvent( 'MERCHANT_CLOSED')
	end
end

function Profit:MERCHANT_CLOSED()
	self:Debug( 'MERCHANT_CLOSED')
	button.frame:SetScript( 'OnUpdate', nil)
	self:SetRate( 0)
	self:UnregisterEvent( 'BAG_UPDATE' )
	self:UnregisterEvent( 'MERCHANT_CLOSED')
end

function Profit:BAG_UPDATE()
	self:Debug( 'BAG_UPDATE')
	self:AUTO_PROFIT_UPDATE()
end

function Profit:AUTO_PROFIT_UPDATE()
	self:Debug( 'AUTO_PROFIT_UPDATE')
	if self.db.profile.autosell then
		self:SetRate( 0)
		button.frame:Hide()
	else
		button.frame:Show()
		if self:IsSpin( 'profit') and GetProfit() > 0 then
			self:SetRate()
		end
	end
end

function Profit:IsSpin( spin)
	return self.db.profile.spin == spin
end

function Profit:SetRate( rate)
	rotation = 0
	rotrate = rate or DEFAULT_SPIN_RATE
	self:Debug( 'SetRate', rotrate)
end

function Profit:SetProficiency( prof)
	PROFICIENCIES = prof
end

--[[---------------------------------------------------------------------------------
Profile
------------------------------------------------------------------------------------]]
function Profit:ListProfiles()
	self:Print( L.AvailableProfiles)
	local current = self.db:GetCurrentProfile()
	for _,name in ipairs(self.db:GetProfiles()) do
		self:Print( format( (name == current) and ' - |cFFFFFF00%s|r' or ' - %s', name))
	end
end

function Profit:OnNewProfile( event, db, name)
--	self:Update()
	self:Print( L.ProfileCreated:format( name))
end

function Profit:OnProfileChanged( event, db, name)
--	self:Update()
	self:Print( L.ProfileLoaded:format( name))
end

function Profit:OnProfileCopied( event, db, name)
--	self:Update()
	self:Print( L.ProfileCopied:format( name))
end

function Profit:OnProfileReset( event, db)
--	self:Update()
	self:Print( L.ProfileReset:format( db:GetCurrentProfile()))
end

function Profit:OnProfileDeleted( event, db, name)
	self:Print( L.ProfileDeleted:format( name))
end
